@@ -1,4 +1,3 @@
-#!/usr/local/bin/python2.7
 # -*- coding: utf-8 -*-
 
 # file legacy_lyxpreview2ppm.py
@@ -36,12 +35,15 @@
 #   the images correctly on the screen.
 
 # The script uses several external programs and files:
+# * python 2.4 or later (subprocess module);
 # * A latex executable;
 # * preview.sty;
 # * dvips;
 # * gs;
 # * pdflatex (optional);
 # * pnmcrop (optional).
+# * pdftocairo (optional).
+# * epstopdf (optional).
 
 # preview.sty is part of the preview-latex project
 #   http://preview-latex.sourceforge.net/
@@ -50,15 +52,18 @@
 
 # What does this script do?
 # [legacy_conversion]
+# 0) Process command-line arguments
+# [legacy_conversion_step1]
 # 1) Call latex to create a DVI file from LaTeX
 # [legacy_conversion_step2]
 # 2) Call dvips to create one PS file for each DVI page
-# 3) If dvips fails look for PDF and call gs to produce bitmaps
-# 4) Otherwise call gs on each PostScript file to produce bitmaps
+# [legacy_conversion_step3]
+# 3) If dvips fails look for PDF and call pdftocairo or gs to produce bitmaps
+# 4) Otherwise call pdftocairo or gs on each PostScript file to produce bitmaps
 # [legacy_conversion_pdflatex]
 # 5) Keep track of pages on which gs failed and pass them to pdflatex
-# 6) Call gs on the PDF output from pdflatex to produce bitmaps
-# 7) Extract and write to file (or return to lyxpreview2bitmap) 
+# 6) Call pdftocairo or gs on the PDF output from pdflatex to produce bitmaps
+# 7) Extract and write to file (or return to lyxpreview2bitmap)
 #    metrics from both methods (standard and pdflatex)
 
 # The script uses the old dvi->ps->png conversion route,
@@ -67,26 +72,24 @@
 # This script also generates bitmaps from PDF created by a call to
 # lyxpreview2bitmap.py passing "pdflatex" to the CONVERTER parameter
 # (step 3).
-# Finally, there's also has a fallback method based on pdflatex, which 
+# Finally, there's also has a fallback method based on pdflatex, which
 # is required in certain cases, if hyperref is active for instance,
 # (step 5, 6).
 # If possible, dvipng should be used, as it's much faster.
+# If possible, the script will use pdftocairo instead of gs,
+# as it's much faster and gives better results.
 
 import glob, os, pipes, re, string, sys
 
-from lyxpreview_tools import copyfileobj, error, find_exe, \
-     find_exe_or_terminate, make_texcolor, mkstemp, run_command, warning, \
-     write_metrics_info, filter_pages, join_metrics_and_rename
+from lyxpreview_tools import check_latex_log, copyfileobj, error, filter_pages,\
+     find_exe, find_exe_or_terminate, join_metrics_and_rename, latex_commands, \
+     latex_file_re, make_texcolor, mkstemp, pdflatex_commands, progress, \
+     run_command, run_latex, warning, write_metrics_info
 
-# Pre-compiled regular expression.
-latex_file_re = re.compile("\.tex$")
-
-# PATH environment variable
-path  = string.split(os.environ["PATH"], os.pathsep)
 
 def usage(prog_name):
-    return "Usage: %s <latex file> <dpi> ppm <fg color> <bg color>\n"\
-           "\twhere the colors are hexadecimal strings, eg 'faf0e6'"\
+    return "Usage: %s <latex file> <dpi> ppm <fg color> <bg color>\n" \
+           "\twhere the colors are hexadecimal strings, eg 'faf0e6'" \
            % prog_name
 
 # Returns a list of tuples containing page number and ascent fraction
@@ -115,13 +118,16 @@ def legacy_extract_metrics_info(log_file):
                 error("Unexpected data in %s\n%s" % (log_file, line))
 
             if snippet:
-                ascent  = string.atoi(match.group(2))
-                descent = string.atoi(match.group(3))
+                ascent  = string.atof(match.group(2))
+                descent = string.atof(match.group(3))
 
                 frac = 0.5
-                if ascent >= 0 and descent >= 0:
-                    ascent = float(ascent) + tp_ascent
-                    descent = float(descent) - tp_descent
+                if ascent == 0 and descent == 0:
+                    # This is an empty image, forbid its display
+                    frac = -1.0
+                elif ascent >= 0 or descent >= 0:
+                    ascent = ascent + tp_ascent
+                    descent = descent - tp_descent
 
                     if abs(ascent + descent) > 0.1:
                         frac = ascent / (ascent + descent)
@@ -141,10 +147,10 @@ def legacy_extract_metrics_info(log_file):
         # the calling function will act on the value of 'success'.
         warning('Warning in legacy_extract_metrics_info! Unable to open "%s"' % log_file)
         warning(`sys.exc_type` + ',' + `sys.exc_value`)
-		
+
     if success == 0:
         error("Failed to extract metrics info from %s" % log_file)
-        
+
     return results
 
 def extract_resolution(log_file, dpi):
@@ -194,42 +200,49 @@ def extract_resolution(log_file, dpi):
     return dpi * (10.0 / fontsize) * (1000.0 / magnification)
 
 
-def legacy_latex_file(latex_file, fg_color, bg_color, bg_color_gr):
-    use_preview_dvi_re = re.compile("(\s*\\\\usepackage\[[^]]+)(dvips\]{preview})")
-    use_preview_pdf_re = re.compile("(\s*\\\\usepackage\[[^]]+)(pdftex\]{preview})")
+def legacy_latex_file(latex_file, fg_color, bg_color):
+    use_preview_re = re.compile(r"\s*\\usepackage\[([^]]+)\]{preview}")
+    fg_color_gr = make_texcolor(fg_color, True)
+    bg_color_gr = make_texcolor(bg_color, True)
 
     tmp = mkstemp()
 
     success = 0
     try:
-        for line in open(latex_file, 'r').readlines():
-            match = use_preview_dvi_re.match(line)
-            if match == None:
-                match = use_preview_pdf_re.match(line)
-                if match == None:
-                    tmp.write(line)
-                    continue
-                success = 1
-                tmp.write("  \\usepackage{color}\n" \
-                      "  \\pagecolor[rgb]{%s}\n" \
-                      "%s\n" \
-                      % (bg_color_gr, match.group()))
-                continue
-
-            success = 1
-            tmp.write("%stightpage,%s\n" \
-                      "  \\AtBeginDocument{\\AtBeginDvi{%%\n" \
-                      "  \\special{!userdict begin/bop-hook{//bop-hook exec\n" \
-                      "  <%s%s>{255 div}forall setrgbcolor\n" \
-                      "  clippath fill setrgbcolor}bind def end}}}\n" \
-                      % (match.group(1), match.group(2), fg_color, bg_color))
-
+        f = open(latex_file, 'r')
     except:
         # Unable to open the file, but do nothing here because
         # the calling function will act on the value of 'success'.
         warning('Warning in legacy_latex_file! Unable to open "%s"' % latex_file)
         warning(`sys.exc_type` + ',' + `sys.exc_value`)
 
+    for line in f.readlines():
+        if success:
+            tmp.write(line)
+            continue
+        match = use_preview_re.match(line)
+        if match == None:
+            tmp.write(line)
+            continue
+        success = 1
+        # Package order: color should be loaded before preview
+        # Preview options: add the options lyx and tightpage
+        tmp.write(r"""
+\usepackage{color}
+\definecolor{fg}{rgb}{%s}
+\definecolor{bg}{rgb}{%s}
+\pagecolor{bg}
+\usepackage[%s,tightpage]{preview}
+\makeatletter
+\def\t@a{cmr}
+\if\f@family\t@a
+\IfFileExists{lmodern.sty}{\usepackage{lmodern}}{\usepackage{ae,aecompl}}
+\fi
+\g@addto@macro\preview{\begingroup\color{bg}\special{ps::clippath fill}\color{fg}}
+\g@addto@macro\endpreview{\endgroup}
+\makeatother
+""" % (fg_color_gr, bg_color_gr, match.group(1)))
+
     if success:
         copyfileobj(tmp, open(latex_file,"wb"), 1)
 
@@ -250,12 +263,13 @@ def crop_files(pnmcrop, basename):
 
 
 def legacy_conversion(argv, skipMetrics = False):
-    latex_commands = ["latex", "pplatex", "platex", "latex2e"]
     # Parse and manipulate the command line arguments.
     if len(argv) == 7:
-        latex_commands = [argv[6]]
+        latex = [argv[6]]
     elif len(argv) != 6:
         error(usage(argv[0]))
+    else:
+        latex = None
 
     dir, latex_file = os.path.split(argv[1])
     if len(dir) != 0:
@@ -267,94 +281,161 @@ def legacy_conversion(argv, skipMetrics = False):
 
     fg_color = argv[4]
     bg_color = argv[5]
-    bg_color_gr = make_texcolor(argv[5], True)
 
     # External programs used by the script.
-    latex = find_exe_or_terminate(latex_commands, path)
+    latex = find_exe_or_terminate(latex or latex_commands)
 
-    # Move color information into the latex file.
-    if not legacy_latex_file(latex_file, fg_color, bg_color, bg_color_gr):
-        error("Unable to move color info into the latex file")
+    pdf_output = latex in pdflatex_commands
+
+    return legacy_conversion_step1(latex_file, dpi, output_format, fg_color,
+        bg_color, latex, pdf_output, skipMetrics)
+
+
+# Add color info to the latex file, since ghostscript doesn't
+# have the option to set foreground and background colors on
+# the command line. Run the resulting file through latex.
+def legacy_conversion_step1(latex_file, dpi, output_format, fg_color, bg_color,
+                            latex, pdf_output = False, skipMetrics = False):
+
+    # Move color information, lyx and tightpage options into the latex file.
+    if not legacy_latex_file(latex_file, fg_color, bg_color):
+        error("""Unable to move the color information, and the lyx and tightpage
+            options of preview-latex, into the latex file""")
 
     # Compile the latex file.
-    latex_call = '%s "%s"' % (latex, latex_file)
+    latex_status, latex_stdout = run_latex(latex, latex_file)
+    if latex_status:
+        progress("Will try to recover from %s failure" % latex)
 
-    latex_status, latex_stdout = run_command(latex_call)
-    if latex_status != None:
-        warning("%s had problems compiling %s" \
-              % (os.path.basename(latex), latex_file))
+    if pdf_output:
+        return legacy_conversion_step3(latex_file, dpi, output_format, True, skipMetrics)
+    else:
+        return legacy_conversion_step2(latex_file, dpi, output_format, skipMetrics)
 
-    return legacy_conversion_step2(latex_file, dpi, output_format, skipMetrics)
-
-# Creates a new LaTeX file from the original with pages specified in 
+# Creates a new LaTeX file from the original with pages specified in
 # failed_pages, pass it through pdflatex and updates the metrics
 # from the standard legacy route
-def legacy_conversion_pdflatex(latex_file, failed_pages, legacy_metrics, gs, 
-    gs_device, gs_ext, alpha, resolution, output_format):
-        
+def legacy_conversion_pdflatex(latex_file, failed_pages, legacy_metrics,
+    use_pdftocairo, conv, gs_device, gs_ext, alpha, resolution, output_format):
+
+    error_count = 0
+
     # Search for pdflatex executable
-    pdflatex = find_exe(["pdflatex"], path)
+    pdflatex = find_exe(["pdflatex"])
     if pdflatex == None:
         warning("Can't find pdflatex. Some pages failed with all the possible routes.")
+        failed_pages = []
     else:
         # Create a new LaTeX file from the original but only with failed pages
         pdf_latex_file = latex_file_re.sub("_pdflatex.tex", latex_file)
         filter_pages(latex_file, pdf_latex_file, failed_pages)
-            
+
         # pdflatex call
-        pdflatex_call = '%s "%s"' % (pdflatex, pdf_latex_file)
-        pdflatex_status, pdflatex_stdout = run_command(pdflatex_call)
-            
+        error_pages = []
+        pdflatex_status, pdflatex_stdout = run_latex(pdflatex, pdf_latex_file)
+        if pdflatex_status:
+            error_pages = check_latex_log(latex_file_re.sub(".log", pdf_latex_file))
+
         pdf_file = latex_file_re.sub(".pdf", pdf_latex_file)
-            
-        # GhostScript call to produce bitmaps
-        gs_call = '%s -dNOPAUSE -dBATCH -dSAFER -sDEVICE=%s ' \
-                    '-sOutputFile="%s%%d.%s" ' \
-                    '-dGraphicsAlphaBit=%d -dTextAlphaBits=%d ' \
-                    '-r%f "%s"' \
-                    % (gs, gs_device, latex_file_re.sub("", pdf_latex_file), \
-                        gs_ext, alpha, alpha, resolution, pdf_file)
-        gs_status, gs_stdout = run_command(gs_call)
-        if gs_status != None:
+        latex_file_root = latex_file_re.sub("", pdf_latex_file)
+
+        # Converter call to produce bitmaps
+        if use_pdftocairo:
+            conv_call = '%s -png -transp -r %d "%s" "%s"' \
+                        % (conv, resolution, pdf_file, latex_file_root)
+            conv_status, conv_stdout = run_command(conv_call)
+            if not conv_status:
+                seqnum_re = re.compile("-([0-9]+)")
+                for name in glob.glob("%s-*.png" % latex_file_root):
+                    match = seqnum_re.search(name)
+                    if match != None:
+                        new_name = seqnum_re.sub(str(int(match.group(1))), name)
+                        os.rename(name, new_name)
+        else:
+            conv_call = '%s -dNOPAUSE -dBATCH -dSAFER -sDEVICE=%s ' \
+                        '-sOutputFile="%s%%d.%s" ' \
+                        '-dGraphicsAlphaBit=%d -dTextAlphaBits=%d ' \
+                        '-r%f "%s"' \
+                        % (conv, gs_device, latex_file_root, \
+                            gs_ext, alpha, alpha, resolution, pdf_file)
+            conv_status, conv_stdout = run_command(conv_call)
+
+        if conv_status:
             # Give up!
             warning("Some pages failed with all the possible routes")
+            failed_pages = []
         else:
             # We've done it!
             pdf_log_file = latex_file_re.sub(".log", pdf_latex_file)
             pdf_metrics = legacy_extract_metrics_info(pdf_log_file)
-                
+
+            # Invalidate metrics for pages that produced errors
+            if len(error_pages) > 0:
+                for index in error_pages:
+                    pdf_metrics.pop(index - 1)
+                    pdf_metrics.insert(index - 1, (index, -1.0))
+                    error_count += 1
+
             original_bitmap = latex_file_re.sub("%d." + output_format, pdf_latex_file)
             destination_bitmap = latex_file_re.sub("%d." + output_format, latex_file)
-                
+
             # Join the metrics with the those from dvips and rename the bitmap images
-            join_metrics_and_rename(legacy_metrics, pdf_metrics, failed_pages, 
+            join_metrics_and_rename(legacy_metrics, pdf_metrics, failed_pages,
                 original_bitmap, destination_bitmap)
 
+    return error_count
 
+
+# The file has been processed through latex and we expect dvi output.
+# Run dvips, taking note whether it was successful.
 def legacy_conversion_step2(latex_file, dpi, output_format, skipMetrics = False):
     # External programs used by the script.
-    dvips   = find_exe_or_terminate(["dvips"], path)
-    gs      = find_exe_or_terminate(["gswin32c", "gswin64c", "gs"], path)
-    pnmcrop = find_exe(["pnmcrop"], path)
+    dvips   = find_exe_or_terminate(["dvips"])
 
     # Run the dvi file through dvips.
     dvi_file = latex_file_re.sub(".dvi", latex_file)
     ps_file  = latex_file_re.sub(".ps",  latex_file)
-    pdf_file  = latex_file_re.sub(".pdf", latex_file)
 
     dvips_call = '%s -i -o "%s" "%s"' % (dvips, ps_file, dvi_file)
     dvips_failed = False
 
     dvips_status, dvips_stdout = run_command(dvips_call)
-    if dvips_status != None:
+    if dvips_status:
         warning('Failed: %s %s ... looking for PDF' \
             % (os.path.basename(dvips), dvi_file))
         dvips_failed = True
 
-    # Extract resolution data for gs from the log file.
+    return legacy_conversion_step3(latex_file, dpi, output_format, dvips_failed, skipMetrics)
+
+
+# Either latex and dvips have been run and we have a ps file, or
+# pdflatex has been run and we have a pdf file. Proceed with pdftocairo or gs.
+def legacy_conversion_step3(latex_file, dpi, output_format, dvips_failed, skipMetrics = False):
+    # External programs used by the script.
+    gs      = find_exe_or_terminate(["gswin32c", "gswin64c", "gs"])
+    pnmcrop = find_exe(["pnmcrop"])
+    pdftocairo = find_exe(["pdftocairo"])
+    epstopdf   = find_exe(["epstopdf"])
+    use_pdftocairo = pdftocairo != None and output_format == "png"
+    if use_pdftocairo:
+        conv = pdftocairo
+    else:
+        conv = gs
+
+    # Files to process
+    pdf_file  = latex_file_re.sub(".pdf", latex_file)
+    ps_file  = latex_file_re.sub(".ps",  latex_file)
+
+    # The latex file name without extension
+    latex_file_root = latex_file_re.sub("", latex_file)
+
+    # Extract resolution data for the converter from the log file.
     log_file = latex_file_re.sub(".log", latex_file)
     resolution = extract_resolution(log_file, dpi)
 
+    # Check whether some pages produced errors
+    error_pages = check_latex_log(log_file)
+
     # Older versions of gs have problems with a large degree of
     # anti-aliasing at high resolutions
     alpha = 4
@@ -369,53 +450,96 @@ def legacy_conversion_step2(latex_file, dpi, output_format, skipMetrics = False)
 
     # Extract the metrics from the log file
     legacy_metrics = legacy_extract_metrics_info(log_file)
-    
+
     # List of pages which failed to produce a correct output
     failed_pages = []
-    
+
     # Generate the bitmap images
     if dvips_failed:
         # dvips failed, maybe there's a PDF, try to produce bitmaps
-        gs_call = '%s -dNOPAUSE -dBATCH -dSAFER -sDEVICE=%s ' \
-                  '-sOutputFile="%s%%d.%s" ' \
-                  '-dGraphicsAlphaBit=%d -dTextAlphaBits=%d ' \
-                  '-r%f "%s"' \
-                  % (gs, gs_device, latex_file_re.sub("", latex_file), \
-                     gs_ext, alpha, alpha, resolution, pdf_file)
+        if use_pdftocairo:
+            conv_call = '%s -png -transp -r %d "%s" "%s"' \
+                        % (pdftocairo, resolution, pdf_file, latex_file_root)
 
-        gs_status, gs_stdout = run_command(gs_call)
-        if gs_status != None:
-            error("Failed: %s %s" % (os.path.basename(gs), ps_file))
+            conv_status, conv_stdout = run_command(conv_call)
+            if not conv_status:
+                seqnum_re = re.compile("-([0-9]+)")
+                for name in glob.glob("%s-*.png" % latex_file_root):
+                    match = seqnum_re.search(name)
+                    if match != None:
+                        new_name = seqnum_re.sub(str(int(match.group(1))), name)
+                        os.rename(name, new_name)
+        else:
+            conv_call = '%s -dNOPAUSE -dBATCH -dSAFER -sDEVICE=%s ' \
+                      '-sOutputFile="%s%%d.%s" ' \
+                      '-dGraphicsAlphaBit=%d -dTextAlphaBits=%d ' \
+                      '-r%f "%s"' \
+                      % (gs, gs_device, latex_file_root, \
+                         gs_ext, alpha, alpha, resolution, pdf_file)
+
+            conv_status, conv_stdout = run_command(conv_call)
+
+        if conv_status:
+            error("Failed: %s %s" % (os.path.basename(conv), pdf_file))
     else:
-        # Model for calling gs on each file
-        gs_call = '%s -dNOPAUSE -dBATCH -dSAFER -sDEVICE=%s ' \
-                  '-sOutputFile="%s%%d.%s" ' \
-                  '-dGraphicsAlphaBit=%d -dTextAlphaBits=%d ' \
-                  '-r%f "%%s"' \
-                  % (gs, gs_device, latex_file_re.sub("", latex_file), \
-                     gs_ext, alpha, alpha, resolution)
-        
+        # Model for calling the converter on each file
+        if use_pdftocairo and epstopdf != None:
+            conv_call = '%s -png -transp -singlefile -r %d "%%s" "%s%%d"' \
+                        % (pdftocairo, resolution, latex_file_root)
+        else:
+            conv_call = '%s -dNOPAUSE -dBATCH -dSAFER -sDEVICE=%s ' \
+                        '-sOutputFile="%s%%d.%s" ' \
+                        '-dGraphicsAlphaBit=%d -dTextAlphaBits=%d ' \
+                        '-r%f "%%s"' \
+                        % (gs, gs_device, latex_file_root, \
+                           gs_ext, alpha, alpha, resolution)
+
         i = 0
         # Collect all the PostScript files (like *.001, *.002, ...)
-        ps_files = glob.glob("%s.[0-9][0-9][0-9]" % latex_file_re.sub("", latex_file))
+        ps_files = glob.glob("%s.[0-9][0-9][0-9]" % latex_file_root)
         ps_files.sort()
-        
-        # Call GhostScript for each file
+
+        # Call the converter for each file
         for file in ps_files:
             i = i + 1
-            gs_status, gs_stdout = run_command(gs_call % (i, file))
-            if gs_status != None:
-                # gs failed, keep track of this
+            progress("Processing page %s, file %s" % (i, file))
+            if use_pdftocairo and epstopdf != None:
+                conv_name = "epstopdf"
+                conv_status, conv_stdout = run_command("%s --outfile=%s.pdf %s"
+                                                       % (epstopdf, file, file))
+                if not conv_status:
+                    conv_name = "pdftocairo"
+                    file = file + ".pdf"
+                    conv_status, conv_stdout = run_command(conv_call % (file, i))
+            else:
+                conv_name = "ghostscript"
+                conv_status, conv_stdout = run_command(conv_call % (i, file))
+
+            if conv_status:
+                # The converter failed, keep track of this
+                warning("%s failed on page %s, file %s" % (conv_name, i, file))
                 failed_pages.append(i)
-    
+
     # Pass failed pages to pdflatex
     if len(failed_pages) > 0:
-        legacy_conversion_pdflatex(latex_file, failed_pages, legacy_metrics, gs, 
-            gs_device, gs_ext, alpha, resolution, output_format)
+        warning("Now trying to obtain failed previews through pdflatex")
+        error_count = legacy_conversion_pdflatex(latex_file, failed_pages,
+            legacy_metrics, use_pdftocairo, conv, gs_device, gs_ext, alpha,
+            resolution, output_format)
+    else:
+        error_count = 0
 
-    # Crop the images
-    if pnmcrop != None:
-        crop_files(pnmcrop, latex_file_re.sub("", latex_file))
+    # Invalidate metrics for pages that produced errors
+    if len(error_pages) > 0:
+        for index in error_pages:
+            if index not in failed_pages:
+                legacy_metrics.pop(index - 1)
+                legacy_metrics.insert(index - 1, (index, -1.0))
+                error_count += 1
+
+    # Crop the ppm images
+    if pnmcrop != None and output_format == "ppm":
+        crop_files(pnmcrop, latex_file_root)
 
     # Allow to skip .metrics creation for custom management
     # (see the dvipng method)
@@ -423,6 +547,8 @@ def legacy_conversion_step2(latex_file, dpi, output_format, skipMetrics = False)
         # Extract metrics info from the log file.
         metrics_file = latex_file_re.sub(".metrics", latex_file)
         write_metrics_info(legacy_metrics, metrics_file)
+        if error_count:
+            warning("Failed to produce %d preview snippet(s)" % error_count)
 
     return (0, legacy_metrics)
 
